@@ -2,6 +2,7 @@ package Dancer::Plugin::Database;
 
 use strict;
 use Dancer::Plugin;
+use Dancer::Config;
 use DBI;
 use Dancer::Plugin::Database::Handle;
 
@@ -11,7 +12,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.00';
+our $VERSION = '1.11_01';
 
 my $settings = undef;
 
@@ -26,26 +27,42 @@ my $def_handle = {};
 register database => sub {
     my $arg = shift;
 
-    my $name;
-    my $handle;
-
     _load_db_settings() if (!$settings);
 
-    # Update settings from configuration file with those from application
-    if ( ref $arg eq 'HASH' ) {
-        for my $key ( keys %$arg ) {
-            $settings->{$key} = $arg->{$key};
+    # The key to use to store this handle in %handles.  This will be either the
+    # name supplied to database(), the hashref supplied to database() (thus, as
+    # long as the same hashref of settings is passed, the same handle will be
+    # reused) or $def_handle if database() is called without args:
+    my $handle_key;
+    my $conn_details; # connection settings to use.
+    my $handle;
+
+
+    # Accept a hashref of settings to use, if desired.  If so, we use this
+    # hashref to look for the handle, too, so as long as the same hashref is
+    # passed to the database() keyword, we'll reuse the same handle:
+    if (ref $arg eq 'HASH') {
+        $handle_key = $arg;
+        $conn_details = $arg;
+    } else {
+        $handle_key = defined $arg ? $arg : $def_handle;
+        $conn_details = _get_settings($arg);
+        if (!$conn_details) {
+            Dancer::Logger::error(
+                "No DB settings for " . ($arg || "default connection")
+            );
+            return;
         }
     }
-    else {
-        $name     = $arg;
-        $handle   = defined($name) ? $handles{$name} : $def_handle;
-        $settings = _get_settings($name);
-    }
 
+    # OK, see if we have a matching handle
+    $handle = $handles{$handle_key} || {};
+    
     if ($handle->{dbh}) {
-        if (time - $handle->{last_connection_check}
-            < $settings->{connection_check_threshold}) {
+        if ($conn_details->{connection_check_threshold} &&
+            time - $handle->{last_connection_check}
+            < $conn_details->{connection_check_threshold}) 
+        {
             return $handle->{dbh};
         } else {
             if (_check_connection($handle->{dbh})) {
@@ -56,19 +73,15 @@ register database => sub {
                     "Database connection went away, reconnecting"
                 );
                 if ($handle->{dbh}) { $handle->{dbh}->disconnect; }
-                return $handle->{dbh}= _get_connection($settings);
+                return $handle->{dbh}= _get_connection($conn_details);
+
             }
         }
     } else {
         # Get a new connection
-        if (!$settings) {
-            Dancer::Logger::error(
-                "No DB settings named $name, so cannot connect"
-            );
-            return;
-        }
-        if ($handle->{dbh} = _get_connection($settings)) {
+        if ($handle->{dbh} = _get_connection($conn_details)) {
             $handle->{last_connection_check} = time;
+            $handles{$handle_key} = $handle;
             return $handle->{dbh};
         } else {
             return;
@@ -109,6 +122,28 @@ sub _get_connection {
         $dsn .= ':' . join(';', @extra_args) if @extra_args;
     }
 
+    # If the app is configured to use UTF-8, the user will want text from the
+    # database in UTF-8 to Just Work, so if we know how to make that happen, do
+    # so, unless they've set the auto_utf8 plugin setting to a false value.
+    my $app_charset = Dancer::Config::setting('charset');
+    my $auto_utf8 = exists $settings->{auto_utf8} ?  $settings->{auto_utf8} : 1;
+    if (lc $app_charset eq 'utf-8' && $auto_utf8) {
+        
+        # The option to pass to the DBI->connect call depends on the driver:
+        my %param_for_driver = (
+            SQLite => 'sqlite_unicode',
+            mysql  => 'mysql_enable_utf8',
+            Pg     => 'pg_enable_utf8',
+        );
+        if (my $param = $param_for_driver{ $settings->{driver} }) {
+            Dancer::Logger::debug(
+                "Adding $param to DBI connection params to enable UTF-8 support"
+            );
+            $settings->{dbi_params}{$param} = 1;
+        }
+    }
+
+
     my $dbh = DBI->connect($dsn, 
         $settings->{username}, $settings->{password}, $settings->{dbi_params}
     );
@@ -159,8 +194,6 @@ sub _get_settings {
     my $return_settings;
 
     # If no name given, just return the default settings
-    # (Take a copy and remove the connections key, so we have only the main
-    # connection details)
     if (!defined $name) {
         $return_settings = { %$settings };
     } else {
@@ -180,12 +213,10 @@ sub _get_settings {
         }
     }
 
-    # We should have soemthing to return now; remove any unrelated connections
-    # (only needed if this is the default connection), and make sure we have a
+    # We should have soemthing to return now; make sure we have a
     # connection_check_threshold, then return what we found.  In previous
     # versions the documentation contained a typo mentioning
     # connectivity-check-threshold, so support that as an alias.
-    delete $return_settings->{connections};
     if (exists $return_settings->{'connectivity-check-threshold'}
         && !exists $return_settings->{connection_check_threshold})
     {
@@ -219,6 +250,12 @@ sub _get_settings {
         database->quick_insert('people', { name => params->{name} });
     };
 
+    get '/users/:id' => sub {
+        template 'display_user', {
+            person => database->quick_select('users', { id => params->{id} }),
+        };
+    };
+
     dance;
 
 Database connection details are read from your Dancer application config - see
@@ -314,6 +351,17 @@ details to override any in the config file at runtime if desired, for instance:
 
 (Thanks to Alan Haggai for this feature.)
 
+=head1 AUTOMATIC UTF-8 SUPPORT
+
+As of version 1.20, if your application is configured to use UTF-8 (you've
+defined the C<charset> setting in your app config as C<UTF-8>) then support for
+UTF-8 for the database connection will be enabled, if we know how to do so for
+the database driver in use.
+
+If you do not want this behaviour, set C<auto_utf8> to a false value when
+providing the connection details.
+
+
 
 =head1 GETTING A DATABASE HANDLE
 
@@ -332,7 +380,7 @@ You can also pass a hashref of settings if you wish to provide settings at
 runtime.
 
 
-=head1 CONVENIENCE FEATURES (quick_update, quick_insert, quick_delete)
+=head1 CONVENIENCE FEATURES (quick_select, quick_update, quick_insert, quick_delete)
 
 The handle returned by the C<database> keyword is a
 L<Dancer::Plugin::Database::Handle> object, which subclasses the C<DBI::db> DBI
@@ -342,9 +390,15 @@ L<Dancer::Plugin::Database::Handle>.
 
 Examples:
 
+  # Quickly fetch the (first) row whose ID is 42 as a hashref:
+  my $row = database->quick_select($table_name, { id => 42 });
+
+  # Fetch all badgers as an array of hashrefs:
+  my @badgers = database->quick_select('animals', { genus => 'Mellivora' });
+
   # Update the row where the 'id' column is '42', setting the 'foo' column to
   # 'Bar':
-  database->quick_update($table_name, { id => 42 }, { foo => 'Bar');
+  database->quick_update($table_name, { id => 42 }, { foo => 'Bar' });
 
   # Insert a new row, using a named connection (see above)
   database('connectionname')->quick_insert($table_name, { foo => 'Bar' });
@@ -367,6 +421,9 @@ L<http://github.com/bigpresh/Dancer-Plugin-Database>
 
 Feel free to fork the repo and submit pull requests!
 
+Feedback and bug reports are always appreciated.  Even a quick mail to let me
+know the module is useful to you would be very nice - it's nice to know if code
+is being actively used.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -376,6 +433,10 @@ Franck Cuny
 
 Alan Haggai
 
+Christian Sánchez
+
+Michael Stiller
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
